@@ -88,7 +88,7 @@ class MetadataWriter:
       for locale, label_list in labels.items():
         full_path = os.path.join(
             self._temp_folder.name,
-            '{}_labels_{}.txt'.format(name, locale or 'default'))
+            '{}_labels{}.txt'.format(name, '_' + locale if locale else ''))
         model_util.export_labels(full_path, label_list)
         label_files.append(
             md_info.LabelFileMd(file_path=full_path, locale=locale))
@@ -283,6 +283,9 @@ class BrowserFFTSpec(BaseSpec):
   ])
   def _preprocess(self, x, label):
     """Preprocess the dataset to extract the spectrum."""
+    # Add small Gaussian noise to the input x
+    # to solve the potential "nan" problem of the preprocess_model.
+    x = x + 1e-05 * tf.random.normal(x.shape)
     # x has shape (1, EXPECTED_WAVEFORM_LENGTH)
     spectrum = self._preprocess_model(x)
     # y has shape (1, embedding_len)
@@ -366,7 +369,8 @@ class BrowserFFTSpec(BaseSpec):
                     tflite_filepath,
                     with_metadata=True,
                     export_metadata_json_file=True,
-                    index_to_label=None):
+                    index_to_label=None,
+                    quantization_config=None):
     """Converts the retrained model to tflite format and saves it.
 
     This method overrides the default `CustomModel._export_tflite` method, and
@@ -381,6 +385,7 @@ class BrowserFFTSpec(BaseSpec):
         True, export the metadata in the same directory as tflite model.Used
         only if `with_metadata` is True.
       index_to_label: A list that map from index to label class name.
+      quantization_config: Configuration for post-training quantization.
     """
     combined = self.create_serving_model(model)
 
@@ -388,7 +393,7 @@ class BrowserFFTSpec(BaseSpec):
     model_util.set_batch_size(model, batch_size=1)
 
     model_util.export_tflite(
-        combined, tflite_filepath, quantization_config=None)
+        combined, tflite_filepath, quantization_config=quantization_config)
 
     # Sets batch size back to None to support retraining later.
     model_util.set_batch_size(model, batch_size=None)
@@ -599,7 +604,8 @@ class YAMNetSpec(BaseSpec):
                     tflite_filepath,
                     with_metadata=True,
                     export_metadata_json_file=True,
-                    index_to_label=None):
+                    index_to_label=None,
+                    quantization_config=None):
     """Converts the retrained model to tflite format and saves it.
 
     This method overrides the default `CustomModel._export_tflite` method, and
@@ -615,13 +621,14 @@ class YAMNetSpec(BaseSpec):
         True, export the metadata in the same directory as tflite model. Used
         only if `with_metadata` is True.
       index_to_label: A list that map from index to label class name.
+      quantization_config: Configuration for post-training quantization.
     """
     serving_model = self.create_serving_model(model)
 
     # TODO(b/164229433): Remove SELECT_TF_OPS once changes in the bug are
     # released.
     model_util.export_tflite(
-        serving_model, tflite_filepath, quantization_config=None)
+        serving_model, tflite_filepath, quantization_config=quantization_config)
 
     if with_metadata:
       if not ENABLE_METADATA:
